@@ -3,6 +3,11 @@
     
 """
 
+try:
+    import Cookie as cookies
+except ImportError:
+    import http.cookies
+
 from BaseHTTPServer import BaseHTTPRequestHandler
 from urlparse import parse_qs
 
@@ -65,7 +70,7 @@ class Request(object):
             self.form_data  = parse_qs(self.form_vals)
 
         self._headers   = {}
-        self._cookies   = []
+        self._cookies   = {}
         self.env        = env
 
     def get(self, key):
@@ -105,14 +110,42 @@ class Request(object):
 
     def add_cookie(self, key, value, path='/', expires=None, http_only=True, secure=False):
         """ Adds a `Set-Cookie` header to the response """
-        cookie = "{0}={1}; Path={2}".format(key, value, path)
+        #cookie = "{0}={1}; Path={2}".format(key, value, path)
+        #if expires:
+        #    cookie += "; Expires="+expires
+        #if http_only:
+        #    cookie += "; HttpOnly"
+        #if secure:
+        #    cookie += "; Secure"
+        #self._cookies.append(cookie)
+        cookie = cookies.SimpleCookie()
+        cookie[key] = value
+        c = cookie[key]
+
+        c["path"] = path
         if expires:
-            cookie += "; Expires="+expires
+            c["expires"] = expires
         if http_only:
-            cookie += "; HttpOnly"
+            c["httponly"] = ''
         if secure:
-            cookie += "; Secure"
-        self._cookies.append(cookie)
+            c["secure"] = ''
+
+        self._cookies[key] = cookie
+
+    def del_cookie(self, key):
+        """ Sets a cookie to expire """
+        cookie = cookies.SimpleCookie()
+        cookie.load(self.get("HTTP_COOKIE"))
+        if key in cookie:
+            c = cookie[key]
+            c["expires"] = "Thu, 01 Jan 1970 00:00:00 GMT"
+            self._cookies[key] = cookie
+
+    def set_response_code(self, code):
+        """ Sets the response code for the request """
+        if self._resp_started:
+            raise ResponseStartedError()
+        self._resp_code = code
     
     def redirect(self, url):
         """ Adds a `Location` header to the response and updates the status-code to 302 """
@@ -131,5 +164,7 @@ class Request(object):
         if self._resp_code:
             code = self._resp_code
 
+        cookies = [str(c) for c in self._cookies.values()]
+
         self._resp_started = True
-        self._resp(status(code), _headers(self._headers, self._cookies))
+        self._resp(status(code), _headers(self._headers, cookies))
